@@ -11,7 +11,7 @@ use App::Glacier::Timestamp;
 
 sub new {
     croak "bad number of arguments" unless $#_ >= 4;
-    my ($class, $cmd, $vault, $key, $init, %opts) = @_;
+    my ($class, $cmd, $vault, $key, %opts) = @_;
     my $invalidate = delete $opts{invalidate};
     my $ttl = delete $opts{ttl};
     
@@ -20,7 +20,6 @@ sub new {
     }
     
     return bless { _cmd => $cmd,
-		   _init => $init,
 		   _vault => $vault,
 		   _key => $key,
 		   _job => undef,
@@ -28,9 +27,21 @@ sub new {
 		   _invalidate => $invalidate }, $class;
 }
 
+sub fromdb {
+    my ($class, $cmd, $vault, $key, $job) = @_;
+    return bless { _cmd => $cmd,
+		   _vault => $vault,
+		   _key => $key,
+		   _job => $job }, $class;
+}
+
+sub command { shift->{_cmd} }
+sub glacier { shift->command->glacier }
+sub vault { shift->{_vault} };
+
 sub _get_db {
     my ($self) = @_;
-    return $self->{_cmd}->jobdb();
+    return $self->command->jobdb();
 }
 
 sub _get_job {
@@ -48,21 +59,7 @@ sub _get_job {
 	my $job = $db->retrieve($self->{_key}) unless $self->{_invalidate};
 	if (!$job) {
 	    $self->debug(2, "initiating job $self->{_key}");
-	    my $jid = $self->{_cmd}->glacier_eval(@{$self->{_init}});
-	    if ($self->{_cmd}->lasterr) {
-		if ($self->{_cmd}->lasterr('code') == 404) {
-		    $self->{_cmd}->abend(EX_TEMPFAIL,
-					 $self->{_cmd}->last_error_message
-					 . "\n"
-					 . "Try again later or use the --cached option to see the cached content.")
-		} else {
-		    $self->{_cmd}->abend(EX_FAILURE,
-					 "can't create job: ",
-					 $self->{_cmd}->lasterr('code'),
-					 $self->{_cmd}->last_error_message);
-		}
-	    }
-	    $job = { JobId => $jid, Completed => 0 };
+	    $job = { JobId => $self->init, Completed => 0 };
 	    $db->store($self->{_key}, $job);
 	}
 
@@ -70,18 +67,17 @@ sub _get_job {
 	    || ($self->{_ttl} 
 		&& (time - $job->{CompletionDate}->epoch) > $self->{_ttl})) {
 	    $self->debug(2, "checking status of job $self->{_key}");
-	    my $res = $self->{_cmd}->glacier_eval('describe_job',
-						  $self->{_vault},
-						  $job->{JobId});
-	    if ($self->{_cmd}->lasterr) {
-		if ($self->{_cmd}->lasterr('code') == 404) {
+	    my $res = $self->glacier->Describe_job($self->{_vault},
+						   $job->{JobId});
+	    if ($self->glacier->lasterr) {
+		if ($self->glacier->lasterr('code') == 404) {
 		    $self->debug(2, "job $self->{_key} expired");
 		    $db->delete($self->{_key});
 		    return $self->_get_job;
 		} else {
-		    $self->{_cmd}->abend(EX_UNAVAILABLE,
+		    $self->command->abend(EX_UNAVAILABLE,
 					 "can't describe job $job->{JobId}: ",
-					 $self->{_cmd}->last_error_message);
+					 $self->glacier->last_error_message);
 		}
 	    } elsif (ref($res) ne 'HASH') {
 		croak "describe_job returned wrong datatype (".ref($res).") for \"$job->{JobId}\"";
@@ -97,10 +93,7 @@ sub _get_job {
     return $self->{_job};
 }
 
-sub debug {
-    my $self = shift;
-    $self->{_cmd}->debug(@_);
-}
+sub debug { my $self = shift->command->debug(@_) }
 
 sub id {
     my $self = shift;
@@ -115,37 +108,47 @@ sub get {
     return $job->{$key};
 }
 
+sub as_string { shift->get('JobDescription') }
+
+use overload
+    '""' => \&as_string;
+
 sub is_finished {
     my $self = shift;
-    my $db = $self->_get_db;
     return defined($self->get('StatusCode'));
 }
 
 sub is_completed {
     my $self = shift;
-    my $db = $self->_get_db;
     return ($self->get('StatusCode') || '') eq 'Succeeded';
 }
 
 sub status {
     my $self = shift;
-    my $db = $self->_get_db;
     my $status = $self->get('StatusCode');
     return undef unless defined $status;
     return wantarray ? ($status, $self->get('StatusMessage')) : $status;
 }
 
-sub vault {
-    my $self = shift;
-    return $self->{_vault};
-}
-
 sub delete {
     my $self = shift;
+    if (my $cache = $self->cache_file) {
+	if (-f $cache) {
+	    unlink($cache);
+	}
+    }
     my $db = $self->_get_db;
     $db->delete($self->{_key});
 }
 
+sub cache_file {
+    my $self = shift;
+    my $aid = $self->get('ArchiveId') or return;
+    my $vault = $self->get('VaultARN') or return;
+    $vault =~ s{.*:vaults/}{};
+    return $self->command->archive_cache_filename($vault, $aid);
+}
+
 1;
 
     
